/*
 *
 *
 */
#include "common.h"
//#include "usb.h"
#include "usbhw.h"

/*
 * Max Number of Interfaces
 */
#define USB_N_IF          4

/*
 * select default power setting
 *   0: Bus-powered
 *   1: Self-powered
 */
#define USB_POWER         0

/* bmRequestType.Dir */
#define OUT_HOST_TO_DEVICE     0
#define IN_DEVICE_TO_HOST     1

/*
 * structure for USB endpoint
 */
typedef struct _USB_EP_DATA {
	u_char  *pData;
	u_short Count;
} USB_EP_DATA;

/*
 * functions
 */
void ucore_ResetCore(void);
void ucore_WakeUp_host(void);
void ucore_isr_Ep0(u_long event);
static void ucore_DataInStage(void);
static void ucore_DataOutStage(void);

/*
 * variables
 */
           u_short USB_DeviceStatus;
/*static*/ u_char  USB_DeviceAddress;
           u_char  USB_Configuration;
/*static*/ u_long  USB_EndPointMask;
/*static*/ u_long  USB_EndPointHalt;
/*static*/ u_char  EP0Buf[MY_MAX_PACKET_SIZE];
/*static*/ USB_EP_DATA      EP0Data;
u_char SETUPDAT[8];
int setup_data_dir;
int GotSUD;

/*
 * externs
 */
/* in usbhw.c */
extern void uhw_WakeUp(void);
extern void uhw_SetAddress(u_char adr);
extern void uhw_ClrStallEP(u_char EPNum);
extern u_long uhw_WriteEP(u_char EPNum, u_char *pData, u_long cnt);
extern u_long uhw_ReadEP(u_char EPNum, u_char *pData);


/*
 *  Reset USB Core
 *    Parameters:      None
 *    Return Value:    None
 */
void ucore_ResetCore(void)
{
	USB_DeviceStatus  = USB_POWER;
	USB_DeviceAddress = 0;
	USB_Configuration = 0;

	USB_EndPointMask  = 0x00010001;
	USB_EndPointHalt  = 0x00000000;
}

/*
 *  USB Remote Wakeup Function
 *    Return Value:    None
 */
void ucore_WakeUp_host(void)
{
//	if(USB_DeviceStatus & USB_GETSTATUS_REMOTE_WAKEUP) {
		uhw_WakeUp();
//	}
}

/*
 *  USB Endpoint 0 Event Callback
 *    Parameter:       event
 */
void ucore_isr_Ep0(u_long event)
{
	switch(event) {
	case USB_EVT_SETUP:
			uhw_ReadEP(0x00, SETUPDAT);
			setup_data_dir = (SETUPDAT[0] & 0x80) ? IN_DEVICE_TO_HOST : OUT_HOST_TO_DEVICE;
			GotSUD = TRUE;

			EP0Data.Count = SETUPDAT[6] + (SETUPDAT[7]<<8);
			break;

	case USB_EVT_OUT:
			if(setup_data_dir == OUT_HOST_TO_DEVICE) {
				if(EP0Data.Count) {
					ucore_DataOutStage();
					if(EP0Data.Count == 0) {
//						switch(*((u_short *)&SetupPacket)) {
//						default:
//								goto stall_i;
//						}
						/* make StatusInStage */
						uhw_WriteEP(0x80, NULL, 0);
					}
				}
			} else {
				/* make StatusOutStage */
				uhw_ReadEP(0x00, EP0Buf);
			}
			break;

	case USB_EVT_IN:
			if(setup_data_dir == IN_DEVICE_TO_HOST) {
				if(EP0Data.Count) {
					ucore_DataInStage();
				}
			} else {
				if(USB_DeviceAddress & 0x80) {
					USB_DeviceAddress &= 0x7F;
					uhw_SetAddress(USB_DeviceAddress);
				}
			}
			break;

	case USB_EVT_IN_STALL:
			uhw_ClrStallEP(0x80);
			break;

	case USB_EVT_OUT_STALL:
			uhw_ClrStallEP(0x00);
			break;
	}
}

/*
 *  USB Request - Data In Stage
 *    Parameters:      None (global EP0Data)
 *    Return Value:    None
 */
static void ucore_DataInStage(void)
{
	u_long cnt;

	if(EP0Data.Count > MY_MAX_PACKET_SIZE) {
		cnt = MY_MAX_PACKET_SIZE;
	} else {
		cnt = EP0Data.Count;
	}
	cnt = uhw_WriteEP(0x80, EP0Data.pData, cnt);

	EP0Data.pData += cnt;
	EP0Data.Count -= cnt;
}

/*
 *  USB Request - Data Out Stage
 *    Parameters:      None (global EP0Data)
 *    Return Value:    None
 */
static void ucore_DataOutStage(void)
{
	u_long cnt;

	cnt = uhw_ReadEP(0x00, EP0Data.pData);
	EP0Data.pData += cnt;
	EP0Data.Count -= cnt;
}
